#!/bin/sh

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

cleanup ()
{
  if [ "$1" != "--start" ]
  then
    umount `mount | grep $VMNAME | awk '{print $3}'` 2> /dev/null
    umount `mount | grep $VMNAME | awk '{print $3}'` 2> /dev/null
    umount /mnt/templateManager/$VMTEMPLATE 2> /dev/null
    rmdir /mnt/templateManager/$VMNAME 2> /dev/null
  fi
  rm /tmp/menu.tmp.* memory.tmp.* selected-VG.*.tmp /tmp/select-template.*.tmp \
    /tmp/vg.*.tmp /tmp/vmfs-*.tmp 2>/dev/null
}

cleanup --start

configdir=/etc/xen/VMs
templateFile=xtm.conf
VGTEMPLATES=vg_Xen_Prod_disks
DIRTEMPLATES=/mnt/Images/vm-templates

case $1
in
   "--xdialog" ) XLEFT="--left"; DIALOG=Xdialog;;
   "-X" ) XLEFT="--left"; DIALOG=Xdialog;;
   "-G" ) XLEFT="--left"; DIALOG=gdialog;;
   "" ) DIALOG=dialog;;
   * ) echo -e "Usage: \n xen-template-manager [ --xdialog | -G | -X ]"
       exit 1;;
esac

templateConfig ()
{
   cat $templateFile | grep \"\,\" | grep -v \#
}

vmTemplateMenu ()
{
   echo "
   $DIALOG --title \"Virtual Machine Template\" \\
           --backtitle \"Virtual Machine Template Manager RC1\" \\
           --radiolist \"Select the template that you wish to use\"  0 50 10 \\" > /tmp/select-template.$$.tmp

   cat $templateFile | grep \"\,\" | grep -v \# | awk -F"," '{print $1" "$3" off \\"}' >> /tmp/select-template.$$.tmp

   sh /tmp/select-template.$$.tmp 2> /tmp/menu.tmp.$$
   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac

   TLINE=`cat /tmp/menu.tmp.$$`

   VMTEMPLATE=`cat $templateFile | sed -n "${TLINE},${TLINE}p" | awk -F"," '{print $2}'`
   rm -f /tmp/menu.tmp.$$ /tmp/select-template.$$.tmp 2> /dev/null

   VMTYPE=`cat $templateFile | sed -n "${TLINE},${TLINE}p" | sed -e "s/\"//g" | awk -F"," '{print $4}'`
   TEMPLATESIZE=`du -hs $DIRTEMPLATES/lv-T-$VMTEMPLATE.img | awk '{print $1}'`
}

vmNameDialog ()
{
   $DIALOG --title "Virtual Machine Name" \
           --backtitle "Virtual Machine Template Manager RC1" \
           --inputbox "Enter the name for the Virtual Machine, \
\nthis name cannot contain blank spaces" 0 0 2> /tmp/name.tmp.$$
   retval=$?

   VMNAME=`cat /tmp/name.tmp.$$`
   rm -f /tmp/name.tmp.$$

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac

   VMNAMELENGHT=`echo $VMNAME | wc -c | awk '{print $1}'`
}

vmMemoryDialog ()
{
   $DIALOG --backtitle "Virtual Machine Template Manager RC1" \
        --title "Virtual Machine Memory" \
        --radiolist "This is the physical memory for the \
Virtual Machine,\n the swap memory will be the double" 0 0 17 \
        "64"     "The smallest value for a modern Linux VM" off \
        "128"    "Fine for most Linuxes and simple Windows 98 and 2000" off \
        "256"    "This is the minimum for RHAS4/RHES4" ON \
        "512"    "Half a Gigabyte" off \
        "768"    "Half a Gigabyte" off \
        "1024"   "A Gigabyte" off \
        "2048"   "Two Gigabytes, why so much memory?" off 2>/tmp/memory.tmp.$$
        #"Custom"   "Enter by hand the amount in Megabytes" off 2>/tmp/memory.tmp.$$

   retval=$?
   VMRAM=`cat /tmp/memory.tmp.$$`
   VMSWAP=$(($VMRAM*2))
   rm -f /tmp/memory.tmp.$$

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac
}

vmProcessorsDialog ()
{
   SYSPROCESSORS=`cat /proc/cpuinfo | grep proc | wc -l`

   $DIALOG --title "Virtual Machine VCPUs" \
        --backtitle "Virtual Machine Template Manager RC1" \
        --radiolist "Select the number of processors" 0 0 17 \
        "1"   "One vCPU" on \
        "2"   "Two vCPUs" off \
        "3"   "Tree vCPUs" off \
        "4"   "Four vCPUs" off 2> /tmp/vcpus.tmp.$$

   retval=$?
   VMVCPUS=`cat /tmp/vcpus.tmp.$$ | sed -e "s/\"//g"`
   rm -f /tmp/vcpus.tmp.$$

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac

   if [ $SYSPROCESSORS -le 4 ]
   then
   $DIALOG --title "Virtual Machine VCPUs" \
        --backtitle "Virtual Machine Template Manager RC1" \
           --checklist "Your system has $SYSPROCESSORS processors \n
   and you selected to give $VMVCPUS CPUs to the VM.
   On which processors do you want to run the Virual Machine?" 0 0 8 \
        "0" "Processor 1" off \
        "1" "Processor 2" off \
        "2" "Processor 3" off \
        "3" "Processor 4" on 2> /tmp/phycpus.tmp.$$
   else
   $DIALOG --title "Virtual Machine VCPUs" \
        --backtitle "Virtual Machine Template Manager RC1" \
           --checklist "Your system has $SYSPROCESSORS processors \n
and you selected to give $VMVCPUS CPUs to the VM.
On which processors do you want to run the Virual Machine?" 0 0 8 \
        "0" "Processor 1" off \
        "1" "Processor 2" off \
        "2" "Processor 3" off \
        "3" "Processor 3" on \
        "4" "Processor 4" off \
        "5" "Processor 5" off \
        "6" "Processor 6" off \
        "7" "Processor 8" off 2> /tmp/phycpus.tmp.$$
   fi
   
   retval=$?
   tCPUS=`cat /tmp/phycpus.tmp.$$ | sed -e "s/\"//g"`
   rm -f /tmp/phycpus.tmp.$$

   VMCPUS=`echo "$tCPUS" | sed -e "s/\//-/g"`

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac
}

vmNicsDialogs () {

   PNICS=`ip a  | grep xenbr | awk '{print $2}' | sed -e "s/\://g"`
   NICSNUMBER=`echo "$PNICS" | wc -l`

   echo "$DIALOG --title \"VM Network Interfaces\" \\
         --backtitle \"Virtual Machine Template Manager RC1\" \
        --checklist \"Your system has $NICSNUMBER Bridge interfaces\\n
Select the Interfaces that would be avaiable to the Virual Machine?\" 0 0 8 \\" > /tmp/nics.$$.tmp

   for NIC in $PNICS
   do
      echo "\"$NIC\" \"Xen Bridge\" off \\"
   done >> /tmp/nics.$$.tmp
   echo "2> /tmp/selected-nic.$$.tmp" >> /tmp/nics.$$.tmp

   sh /tmp/nics.$$.tmp
   retval=$?

   rm /tmp/nics.$$.tmp

   VMNICS=`cat /tmp/selected-nic.$$.tmp | sed -e "s/\// /g" | sed -e "s/\"//g"`

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac
}

vmFilesystemsDialog () {

   VGS=` vgscan 2> /dev/null | grep Found | awk '{print $4}' | tr -d \"`

   echo "$DIALOG --title \"Filesystems Configuration\" \\
        --backtitle \"Virtual Machine Template Manager RC1\" \
        --radiolist \"Select the Volume Group for the VM Filesystems\"  0 0 8 \\" > /tmp/selected-VG.$$.tmp

   for VG in $VGS
   do
      echo "\"$VG\" \"Volume Group\" off \\"
   done >> /tmp/selected-VG.$$.tmp
   #echo "2> /tmp/selected-nic.$$.tmp" >> /tmp/selected-VG.$$.tmp

   sh /tmp/selected-VG.$$.tmp 2> /tmp/vg.$$.tmp
   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1 ;;
   esac

   # retval must be null
   retval=""

   VMVG=`cat /tmp/vg.$$.tmp`
   rm /tmp/selected-VG.$$.tmp


   retval=$?
   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1;;
   esac


   # retval must be null
   retval=""
   if [ "$VMTYPE" == "pvm" ]
   then
      $DIALOG --title "Filesystems Configuration" \
        --backtitle "Virtual Machine Template Manager RC1" \
	--checklist "Select the aditional filesystems (besides root FS)"  0 0 8 \
        "/usr" " files" off \
        "/var" "Variable content files" off \
        "/home" "Home of local Users" off \
        "/www" "HTTP Server files" off \
        "/tmp" "Temporary files" off 2> /tmp/filesystems-select.tmp.$$
      retval=$?

      case $retval in
        0) true;;
        1) exit 0;;
        255) exit 1 ;;
      esac
   else
      # retval must be null
      retval=""
      $DIALOG  --title "Disk Drives Configuration"\
         --yesno "The primary disk is about $TEMPLATESIZE large, \n\
Do you want an aditional Disk Drive?" 8 40
      retval=$?
      case $retval in
        0) HVMADDTIONALDISK=true;;
        1) HVMADDTIONALDISK=false;;
        255) exit 1;;
      esac
   fi


   if [ "$VMTYPE" == "pvm" ]
   then
      TVMFS=`cat /tmp/filesystems-select.tmp.$$`

      echo "Fix the output from // to / and create the FS List"

      # Fix the output from "//" to " /" and create the FS List
      if [ "$DIALOG" == "dialog" ]; then
        VMFSLIST=( `echo "root $TVMFS" | sed -e "s/\///g" | sed -e "s/\"//g"` )
      else
        VMFSLIST=( `echo root $TVMFS | sed -e "s/\// /g" | sed -e "s/\"//g"` )
      fi

      rm -f /tmp/filesystems-select.tmp.$$

      for TVM in "${VMFSLIST[@]}"
      do

         # displays the name of the FS and then his size, all this
         # goes to /tmp/vmfs-$$.tmp in a list form

         echo "$DIALOG --title \"Virtual Machine Filesystems\" \\
              --backtitle \"Virtual Machine Template Manager RC1\" \
              --inputbox \"Select the size of the $TVM Filesystem \n 
      G - Gigabytes \n M - Megabytes\" 0 0 5G" > /tmp/filesystems-sizes.$$.tmp

         sh /tmp/filesystems-sizes.$$.tmp 2> /tmp/vmfs-T-$$.tmp
         retval=$?

         TempVar=`cat /tmp/vmfs-T-$$.tmp`
         echo "$TVM $TempVar" >> /tmp/vmfs-$$.tmp

         case $retval in
           0) true;;
           1) exit 0;;
           255) exit 1 ;;
         esac
      done # > /tmp/vmfs-$$.tmp 2>&1
   else
      VMFSLIST=`echo "root $TVMFS" | sed -e "s/\// /g"` 

      if [ "$HVMADDTIONALDISK" == "true" ]
      then
         VMFSLIST="primary secondary"
         rm -f /tmp/filesystems-select.tmp.$$
         echo -n "primary $TEMPLATESIZE secondary " > /tmp/vmfs-$$.tmp
         echo "$DIALOG --title \"Virtual Machine Filesystems\" \\
              --backtitle \"Virtual Machine Template Manager RC1\" \
              --inputbox \"Select the size of the secondary disk\n 
      G - Gigabytes \n M - Megabytes\" 0 0 5G" > /tmp/filesystems-sizes.$$.tmp
         sh /tmp/filesystems-sizes.$$.tmp 2>> /tmp/vmfs-$$.tmp 
         retval=$?
         case $retval in
           0) true;;
           1) exit 0;;
           255) exit 1 ;;
         esac
      else
         VMFSLIST="primary"
         rm -f /tmp/filesystems-select.tmp.$$
         echo -n "primary $TEMPLATESIZE" > /tmp/vmfs-$$.tmp
      fi
   fi

   rm /tmp/filesystems-sizes.$$.tmp 2> /dev/null
   VMFS=`cat /tmp/vmfs-$$.tmp` 2> /dev/null
   rm -f /tmp/vmfs-$$.tmp 2> /dev/null
}

showVMconfigDialog ()
{
   $DIALOG --title "Virtual Machine $VMNAME configuration" \
           $XLEFT --backtitle "Virtual Machine Template Manager RC1" \
           --yesno "Template Name\n
   $VMTEMPLATE\n
VCPUs\n
   $VMVCPUS running on Physical CPUS #$VMCPUS\n
Memory \n
   $VMRAM Physical\n
   $VMSWAP Swap memory\n
Network Interfaces\n
   $VMNICS\n
Filesystems\n
   $VMFS\n\n
   Proceed with Virtual Machine creation?\n\n" 0 0

   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1;;
   esac

}

hvmHeaderConfigFile ()
{
echo "
import os, re
arch = os.uname()[4]
if re.search('64', arch):
    arch_libdir = 'lib64'
else:
    arch_libdir = 'lib'

kernel = \"/usr/lib/xen/boot/hvmloader\"
device_model = '/usr/' + arch_libdir + '/xen/bin/qemu-dm'
builder='hvm'

pae=0
acpi=1
apic=0

sdl=0
vnc=1
#vncdisplay=0
vncunused=1
stdvga=0
serial='pty'
boot=\"c\""
}

#
# Generates the config file to start the VM on Xen
# outputs to the stdout, so his output must be
# redirected
#

generateConfigFile ()
{
   [ $VMTYPE == hvm ] && hvmHeaderConfigFile || echo "kernel = \"/boot/vmlinuz-2.6.16-xenU\""
   DISKSNAMES=(hda hdb)

   # Virutal Machine name
   echo "name = \"$VMNAME\""

   # Memory
   [ $VMTYPE == hvm ] && echo "shadow_memory = 8"
   echo "memory = $VMRAM"

   # Disks
   if [ $VMTYPE == hvm ]
   then
      IOEMU="ioemu:"
      COUNT=1
      for CFS in ${VMFSLIST[@]}
      do
         [ $COUNT -lt 2 ] || CFDISKS="${CFDISKS},"
         CFDISKS="${CFDISKS}'phy:/dev/$VMVG/lv-$VMNAME-$CFS,${IOEMU}${DISKSNAMES[$(($COUNT-1))]},w'"
         COUNT=$(($COUNT+1))
      done
      echo "disk = [ ${CFDISKS} ]"
   else
      COUNT=1
      for CFS in ${VMFSLIST[@]}
      do
         CFDISKS="${CFDISKS}'phy:/dev/$VMVG/lv-$VMNAME-$CFS,hda${COUNT},w', "
         COUNT=$(($COUNT+1))
      done
      echo "disk = [ ${CFDISKS}'phy:/dev/$VMVG/lv-$VMNAME-swap,hda${COUNT},w' ]"
   fi

   # CPUs and Virtual CPUs
   echo -e "cpus = \"$VMCPUS\" \nvcpus = \"$VMVCPUS\""

   # Networking VMNICS
   COUNT=1
   for CNIC in $VMNICS
   do
      if [ $VMTYPE == hvm ]
      then
         [ "$CFNICS" == "" ] && CFNICS="'type=ioemu, bridge=$CNIC'" ||  CFNICS="${CFNICS},'type=ioemu, bridge=$CNIC'"
      else
         [ "$CFNICS" == "" ] && CFNICS="'bridge=$CNIC'" ||  CFNICS="${CFNICS},'bridge=$CNIC'"
      fi
      COUNT=$(($COUNT+1))
   done
   echo "vif = [ $CFNICS ]"
   
   # Misc - PAE, ACPI and APIC
   [ $VMTYPE == pvm ] && echo "root = \"/dev/hda1 ro\""
}

generateVM ()
{
  TEMP=`templateConfig | grep $VMTEMPLATE`
  TEMPLATENAME=`echo "$TEMP" | awk -F"," '{print $1}'`
  #VMTYPE=`echo "$TEMP" | awk -F"," '{print $3}'`
}

#
# Creates the Logical Volumes for the Virtual Machine
#
generateVmLVs ()
{
   VMFS2=( `echo $VMFS | sed -e "s/\///g"` )
   #echo "Crea los lv con la variable VMFS2 $VMFS2 y VMFS $VMFS"

   COUNT=0
   for TFS in "${VMFS2[@]}"
   do
      if [ $(($COUNT%2)) == 0 ]
      then
         if [ $VMTYPE == pvm ]
         then
           # *** Paravirtualizada ***

           lvcreate -L${VMFS2[$(($COUNT+1))]} -n lv-$VMNAME-${VMFS2[$COUNT]} $VMVG #> /dev/null 2>&1
           [ ${VMFS2[$COUNT]} == root ] &&  lvcreate -L$(($VMRAM*2)) -n lv-$VMNAME-swap $VMVG \
             #> /dev/null 2>&1

           mkfs.ext3 /dev/$VMVG/lv-$VMNAME-${VMFS2[$COUNT]} 
           [ $COUNT == 0 ] && mkswap /dev/$VMVG/lv-$VMNAME-swap # > /dev/null
         else
           # *** Virtualizada Completa ***
           # Si es el disco primario de una HVM se fija cual es el
           # tamaño de la imagen y crea el disco segun corresponda

           [ ${VMFS2[$COUNT]} == primary ] && lvcreate -L$TEMPLATESIZE -n lv-$VMNAME-${VMFS2[$COUNT]} $VMVG > /dev/null 2>&1 || lvcreate -L${VMFS2[$(($COUNT+1))]} -n lv-$VMNAME-${VMFS2[$COUNT]} $VMVG > /dev/null 2>&1
         fi
      fi
      COUNT=$(($COUNT+1))
   done
}

cloneSuccess ()
{
   $DIALOG --title "Xen Virtual Machine Manager" \
           --backtitle "Template Copy" \
           --yesno "Copy success!" 10 50
   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1;;
   esac
}

cloneFailure ()
{
   $DIALOG --title "Xen Virtual Machine Manager" \
           --backtitle "Template Copy" \
           --yesno "Copy Failed!" 10 50
   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1;;
   esac
}

#
# Copy the template to the LVs created before
#

confirmCopy ()
{
   $DIALOG --title "Xen Virtual Machine Manager" \
           --backtitle "Template Copy" \
           --yesno "A continuacion se copiara el 
template a los discos anteriormente 
creados, este procedimiento no 
tiene salida por pantalla.
\nAguarde al mensaje de confimacion.
\n\nStart the copy?\n\n" 16 50

   retval=$?

   case $retval in
     0) true;;
     1) exit 0;;
     255) exit 1;;
   esac
}

copyVmFromTemplate ()
{
   confirmCopy

   case $VMTYPE
   in
   pvm) 
      for TFSM in ${VMFSLIST[@]}
      do
         if [ $TFSM == root ]
         then
            mkdir -p /mnt/templateManager/$VMTEMPLATE && mkdir -p /mnt/templateManager/$VMNAME
            [ $? ] && mount -o loop $DIRTEMPLATES/lv-T-$VMTEMPLATE.img /mnt/templateManager/$VMTEMPLATE
            [ $? ] && mount /dev/$VMVG/lv-$VMNAME-root /mnt/templateManager/$VMNAME
         else
            mkdir -p /mnt/templateManager/$VMNAME/$TFSM
            [ $? ] && mount /dev/$VMVG/lv-$VMNAME-$TFSM /mnt/templateManager/$VMNAME/$TFSM
         fi
      done

      cp -a /mnt/templateManager/$VMTEMPLATE/* /mnt/templateManager/$VMNAME/ \
         && cloneSuccess || cloneFailure ;;
   hvm)
      dd if=$DIRTEMPLATES/lv-T-$VMTEMPLATE.img of=/dev/$VMVG/lv-$VMNAME-primary > /dev/null 2>&1 \
         && cloneSuccess || cloneFailure ;;
   empty) true ;;
   esac
}

generateFstab ()
{
echo "none                    /dev/pts                devpts  gid=5,mode=620  0 0
none                    /dev/shm                tmpfs   defaults        0 0
none                    /proc                   proc    defaults        0 0
none                    /sys                    sysfs   defaults        0 0" \
  > /mnt/templateManager/$VMNAME/etc/fstab

if [ $VMTYPE == pvm ]
then
  COUNT=1
  for CFS in ${VMFSLIST[@]}
  do
    [ $COUNT -lt 2 ] || CFDISKS="${CFDISKS},"
    [ $CFS == root ] && echo "/dev/hda${COUNT}    / ext3 defaults        0 0" \
       >> /mnt/templateManager/$VMNAME/etc/fstab
    [ $CFS == root ] || echo "/dev/hda${COUNT}    /$CFS ext3 defaults        0 0" \
       >> /mnt/templateManager/$VMNAME/etc/fstab
    COUNT=$(($COUNT+1))
  done
    echo "/dev/hda${COUNT}    swap swap defaults        0 0" \
       >> /mnt/templateManager/$VMNAME/etc/fstab
fi
}


#
# Displays the select template menu - VMTEMPLATE 
#
vmTemplateMenu

#
# Show the dialog for the Virtual Machine Name
# Also checks the minium length of the string - VMNAME
#
VMNAMELENGHT=1
while [ $VMNAMELENGHT -lt 6 ]
do
   vmNameDialog
done

#
# Memory size configuration - VMRAM, VMSWAP
#
vmMemoryDialog

#
# CPUs configuration - VMCPUS, VMVCPUS
#
vmProcessorsDialog

#
# Show all the bridges interfaces - VMNICS, CFNICS
#
vmNicsDialogs

#
# Files Dialog - VMFS, VMVG, VMFSLIST
#
vmFilesystemsDialog

#
# Shows a detail of all configurations for de VM
#
showVMconfigDialog

generateConfigFile > /etc/xen/VMs/vm-$VMNAME.$VMTYPE

generateVmLVs
copyVmFromTemplate
[ $VMTYPE == pvm ] && generateFstab
cleanup

exit 0
